// Generated by dts-bundle-generator v5.9.0

import { Pxth } from 'pxth';
import React from 'react';
import { Dispatch, SetStateAction } from 'react';

/** Object, in which "stocked" calls observers */
export declare type BatchUpdate<T> = {
	/** which paths should be updated */
	paths: Pxth<unknown>[];
	/** path, which triggered subtree update */
	origin: Pxth<unknown>;
	/** all values, which should be sent to observers */
	values: T;
};
export declare type Observer<V> = (message: V) => void;
/**
 * Abstract class of "stocked" proxy
 * By inheriting this class, you can provide custom logic for proxing (casting) values.
 */
export declare abstract class StockProxy<T> {
	/** Path to variable, where proxy is attached. */
	readonly path: Pxth<T>;
	constructor(path: Pxth<T>);
	/** Function, which sets proxied value. It will be callen only if proxied value is changing. */
	abstract setValue: <V>(path: Pxth<V>, value: SetStateAction<V>, defaultSetValue: <U>(path: Pxth<U>, value: SetStateAction<U>) => void, defaultGetValue: <U>(path: Pxth<U>) => U) => void;
	/** Function for watching proxied value. Should return cleanup. */
	abstract watch: <V>(path: Pxth<V>, observer: Observer<V>, defaultWatch: <U>(path: Pxth<U>, observer: Observer<U>) => () => void) => () => void;
	/** Function to access proxied value. */
	abstract getValue: <V>(path: Pxth<V>, defaultGetValue: <U>(path: Pxth<U>) => U) => V;
	/** Function for getting proxied path from normal path. */
	abstract getProxiedPath: <V>(path: Pxth<V>) => Pxth<V>;
	/** Function for getting normal path from proxied path */
	abstract getNormalPath: <V>(path: Pxth<V>) => Pxth<V>;
	/** Activate proxy. After activation, you cannot modify proxy. */
	activate: () => Readonly<this>;
	/** Returns boolean, is proxy active and ready for use or not. */
	isActive: () => boolean;
}
/**
 * Simple example of StockProxy.
 * Proxies values by map. Map is built by this method:
 * {
 *      a: {
 *          b: {
 *              c: "<path to real value, in stock>"
 *          },
 *      }
 * }
 */
export declare class MappingProxy<T> extends StockProxy<T> {
	private readonly proxyMap;
	constructor(mapSource: ProxyMapSource<T>, path: Pxth<T>);
	setValue: <V>(path: Pxth<V>, value: SetStateAction<V>, defaultSetValue: <U>(path: Pxth<U>, value: SetStateAction<U>) => void, defaultGetValue: <U_1>(path: Pxth<U_1>) => U_1) => void;
	watch: <V>(path: Pxth<V>, observer: Observer<V>, defaultWatch: <U>(path: Pxth<U>, observer: Observer<U>) => () => void) => () => void;
	getValue: <V>(path: Pxth<V>, defaultGetValue: <U>(path: Pxth<U>) => U) => V;
	private mapValue;
	getProxiedPath: <V>(path: Pxth<V>) => Pxth<V>;
	getNormalPath: <V>(path: Pxth<V>) => Pxth<V>;
}
export declare type Primitive = string | number | boolean | bigint | symbol | null | undefined;
export declare type ObjectMappingType<T> = {
	[P in keyof T]: ProxyMapSource<T[P]>;
};
export declare type ProxyMapSource<T> = T extends Primitive ? Pxth<T> : ObjectMappingType<T> | Pxth<T>;
export declare class PxthMap<Value> {
	private _values;
	private _keys;
	get: <V>(key: Pxth<V>) => Value;
	set: <V>(key: Pxth<V>, value: Value) => void;
	remove: <V>(key: Pxth<V>) => void;
	has: <V>(key: Pxth<V>) => boolean;
	keys: () => Pxth<unknown>[];
	entries: () => Array<[
		key: Pxth<unknown>,
		value: Value
	]>;
}
export declare type ObserversControl<T> = {
	/** Watch stock value. Returns cleanup function. */
	watch: <V>(path: Pxth<V>, observer: Observer<V>) => () => void;
	/** Watch all stock values. Returns cleanup function. */
	watchAll: (observer: Observer<T>) => () => void;
	/** Check if value is observed or not. */
	isObserved: <V>(path: Pxth<V>) => boolean;
	/** Notify all observers, which are children of specified path */
	notifySubTree: <V>(path: Pxth<V>, values: T) => void;
	/** Notify all observers */
	notifyAll: (values: T) => void;
	/** "stocked" updates values in batches, so you can subscribe to batch updates. Returns cleanup. */
	watchBatchUpdates: (observer: Observer<BatchUpdate<T>>) => () => void;
};
/** Hook, wraps functionality of observers storage (add, remove, notify tree of observers, etc.) */
export declare const useObservers: <T>() => ObserversControl<T>;
export declare type Stock<T extends object> = {
	/** Function for setting value. Deeply sets value, using path to variable. @see https://lodash.com/docs/4.17.15#set */
	setValue: <V>(path: Pxth<V>, value: SetStateAction<V>) => void;
	/** Function for setting all values. */
	setValues: (values: T) => void;
	/** Get actual value from stock. */
	getValue: <V>(path: Pxth<V>) => V;
	/** Get all values from stock */
	getValues: () => T;
	/** Function for resetting values to initial state */
	resetValues: () => void;
	/** Paths to all variables */
	paths: Pxth<T>;
	/** Name, which appears in stocked devtools */
	debugName?: string;
} & Omit<ObserversControl<T>, "notifyAll" | "notifySubTree">;
export declare type StockConfig<T extends object> = {
	initialValues: T;
	debugName?: string;
};
/**
 * Creates stock.
 *
 * Use it only if you want to use several Stock's at the same time.
 *
 * Instead, use `<StockRoot>` component
 *
 * @param config - configuration of Stock.
 */
export declare const useStock: <T extends object>({ initialValues, debugName }: StockConfig<T>) => Stock<T>;
/**
 * Function to access stock, with automatically applying proxy. By default, will pick stock and proxy from context.
 * @param stock - optional argument. Pass it if you don't want to use stock from context.
 * @param proxy - optional argument. Pass it if you don't want to use proxy from context.
 */
export declare const useStockContext: <T extends object>(stock?: Stock<T> | undefined, proxy?: StockProxy<unknown> | undefined) => Stock<T>;
/**
 * Hook, which returns *actual* stock value.
 * This means, it will update component each time when value in stock changes.
 * @param path        - path to variable in stock, deeply gets value. @see https://lodash.com/docs/4.17.15#get
 * @param customStock - optional parameter, if you want to work with custom stock, not received from context. @see useStockContext
 * @param proxy       - optional parameter, if you want to pass custom proxy, not receive it from context. @see useStockContext
 */
export declare const useStockValue: <V, T extends object = object>(path: Pxth<V>, customStock?: Stock<T> | undefined, proxy?: StockProxy<unknown> | undefined) => V;
/**
 * Hook, returns tuple of value and value set action.
 * Returns *actual* value.
 * This means, this hook fires re-render each time value in stock was changed.
 * Similar to standard React's `useState` hook.
 * @param path  - path to variable in stock, deeply gets value. @see https://lodash.com/docs/4.17.15#get
 * @param customStock - optional parameter, if you want to work with custom stock, not received from context. @see useStockContext
 * @param proxy       - optional parameter, if you want to pass custom proxy, not receive it from context. @see useStockContext
 */
export declare const useStockState: <V, T extends object = object>(path: Pxth<V>, customStock?: Stock<T> | undefined, proxy?: StockProxy<unknown> | undefined) => [
	V,
	Dispatch<SetStateAction<V>>
];
export declare const useAllStockValues: <T extends object = object>(customStock?: Stock<T> | undefined, proxy?: StockProxy<unknown> | undefined) => T;
export declare const useMappingProxy: <V>(mapSource: ProxyMapSource<V>, path: Pxth<V>) => MappingProxy<V>;
export declare const StockContext: import("react").Context<Stock<object> | undefined>;
export declare type StockRootProps<Values extends object> = StockConfig<Values> & {
	children: React.ReactNode;
};
/**
 * The main component, which should wrap all code, which uses stock values.
 * Creates stock and puts it in `StockContext`.
 */
export declare const StockRoot: <Values extends object>({ children, ...stockConfig }: StockRootProps<Values>) => JSX.Element;
export declare const ProxyContext: import("react").Context<StockProxy<unknown> | undefined>;
export declare type ProxyProviderProps<V> = {
	proxy: StockProxy<V>;
	children: React.ReactNode | ((path: Pxth<V>) => React.ReactNode);
};
export declare const ProxyProvider: <V>({ proxy, children }: ProxyProviderProps<V>) => JSX.Element;
export declare type ObserverKey = number;
export declare class ObserverArray<V> {
	private readonly observers;
	private counter;
	/**
	 * Calls all observers in the array.
	 * @param message - message, which would be sent to all observers
	 */
	call(message: V): void;
	/**
	 * Registers new observer and returns key, which will be used for removing observer.
	 * @param observer - registers new observer
	 */
	add(observer: Observer<V>): ObserverKey;
	/**
	 * Removes observer from array. Throws an error, if observer with that key not exists.
	 * @param key - key to observer
	 */
	remove(key: ObserverKey): void;
	/**
	 * Does array include at least one observer or not.
	 */
	isEmpty: () => boolean;
}
/**
 * Helper function. Calls `standardCallback` if `proxy` is undefined, or if `path` isn't inner path of `proxy.path` variable.
 * Otherwise, calls `proxiedCallback`.
 * Passes args into function.
 */
export declare const intercept: <T extends (...args: any[]) => any>(proxy: StockProxy<unknown> | undefined, path: Pxth<unknown>, standardCallback: T, proxiedCallback: T, args: Parameters<T>) => ReturnType<T>;

export {};
