var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};

// src/hooks/useStock.ts
import { useCallback as useCallback2, useMemo } from "react";
import cloneDeep from "lodash/cloneDeep";
import isFunction from "lodash/isFunction";
import { createPxth as createPxth2, deepGet as deepGet2, deepSet } from "pxth";

// src/hooks/useDebugStock.ts
import { useEffect } from "react";
import { getPxthSegments } from "pxth";
var stockIndex = -1;
var useDebugStock = /* @__PURE__ */ __name((stock) => {
  useEffect(() => {
    if (window.__STOCKED_DEVTOOLS_HOOK) {
      const currentStockId = ++stockIndex;
      window.__STOCKED_DEVTOOLS_HOOK.raiseEvent("new" /* NEW */, {
        data: stock.getValues(),
        id: currentStockId
      });
      return stock.watchBatchUpdates(
        (data) => window.__STOCKED_DEVTOOLS_HOOK.raiseEvent("update" /* BATCH_UPDATE */, {
          data: __spreadProps(__spreadValues({}, data), {
            origin: getPxthSegments(data.origin)
          }),
          id: currentStockId
        })
      );
    }
    return () => {
    };
  }, [stock]);
}, "useDebugStock");

// src/hooks/useObservers.ts
import { useCallback, useRef as useRef2 } from "react";
import { createPxth, deepGet, isInnerPxth, samePxth } from "pxth";
import invariant2 from "tiny-invariant";

// src/utils/hashPxth.ts
import { getPxthSegments as getPxthSegments2 } from "pxth";
var hashPxth = /* @__PURE__ */ __name((pxth) => JSON.stringify(getPxthSegments2(pxth)), "hashPxth");

// src/typings/PxthMap.ts
var PxthMap = class {
  constructor() {
    this._values = {};
    this._keys = {};
    this.get = /* @__PURE__ */ __name((key) => {
      return this._values[hashPxth(key)];
    }, "get");
    this.set = /* @__PURE__ */ __name((key, value) => {
      const stingifiedKey = hashPxth(key);
      this._values[stingifiedKey] = value;
      this._keys[stingifiedKey] = key;
    }, "set");
    this.remove = /* @__PURE__ */ __name((key) => {
      const stingifiedKey = hashPxth(key);
      delete this._values[stingifiedKey];
      delete this._keys[stingifiedKey];
    }, "remove");
    this.has = /* @__PURE__ */ __name((key) => {
      return hashPxth(key) in this._values;
    }, "has");
    this.keys = /* @__PURE__ */ __name(() => Object.values(this._keys), "keys");
    this.entries = /* @__PURE__ */ __name(() => Object.entries(this._keys).map(([key, value]) => [value, this._values[key]]), "entries");
  }
};
__name(PxthMap, "PxthMap");

// src/utils/ObserverArray.ts
import invariant from "tiny-invariant";
var ObserverArray = class {
  constructor() {
    this.observers = {};
    this.counter = 0;
    this.isEmpty = /* @__PURE__ */ __name(() => Object.values(this.observers).length === 0, "isEmpty");
  }
  call(message) {
    Object.values(this.observers).forEach((observer) => observer(message));
  }
  add(observer) {
    const key = this.counter++;
    this.observers[key] = observer;
    return key;
  }
  remove(key) {
    invariant(
      Object.prototype.hasOwnProperty.call(this.observers, key),
      `Could not remove observer: observer with key ${key} not exists`
    );
    delete this.observers[key];
  }
};
__name(ObserverArray, "ObserverArray");

// src/utils/useLazyRef.ts
import { useRef } from "react";
var useLazyRef = /* @__PURE__ */ __name((initializer) => {
  const ref = useRef(null);
  if (ref.current === null) {
    ref.current = initializer();
  }
  return ref;
}, "useLazyRef");

// src/hooks/useObservers.ts
var useObservers = /* @__PURE__ */ __name(() => {
  const observersMap = useRef2(new PxthMap());
  const batchUpdateObservers = useLazyRef(() => new ObserverArray());
  const batchUpdate = useCallback(
    (update) => {
      batchUpdateObservers.current.call(update);
    },
    [batchUpdateObservers]
  );
  const observeBatchUpdates = useCallback(
    (observer) => batchUpdateObservers.current.add(observer),
    [batchUpdateObservers]
  );
  const stopObservingBatchUpdates = useCallback(
    (key) => batchUpdateObservers.current.remove(key),
    [batchUpdateObservers]
  );
  const observe = useCallback((path, observer) => {
    if (!observersMap.current.has(path)) {
      observersMap.current.set(path, new ObserverArray());
    }
    return observersMap.current.get(path).add(observer);
  }, []);
  const stopObserving = useCallback((path, observerKey) => {
    const currentObservers = observersMap.current.get(path);
    invariant2(currentObservers, "Cannot remove observer from value, which is not observing");
    currentObservers.remove(observerKey);
    if (currentObservers.isEmpty()) {
      observersMap.current.remove(path);
    }
  }, []);
  const watch = useCallback(
    (path, observer) => {
      const key = observe(path, observer);
      return () => stopObserving(path, key);
    },
    [observe, stopObserving]
  );
  const watchAll = useCallback((observer) => watch(createPxth([]), observer), [watch]);
  const watchBatchUpdates = useCallback(
    (observer) => {
      const key = observeBatchUpdates(observer);
      return () => stopObservingBatchUpdates(key);
    },
    [observeBatchUpdates, stopObservingBatchUpdates]
  );
  const isObserved = useCallback((path) => observersMap.current.has(path), []);
  const notifyPaths = useCallback(
    (origin, paths, values) => {
      batchUpdate({ paths, origin, values });
      paths.forEach((path) => {
        const observer = observersMap.current.get(path);
        const subValue = deepGet(values, path);
        observer.call(subValue);
      });
    },
    [batchUpdate]
  );
  const notifySubTree = useCallback(
    (path, values) => {
      const subPaths = observersMap.current.keys().filter(
        (tempPath) => isInnerPxth(path, tempPath) || samePxth(path, tempPath) || isInnerPxth(tempPath, path)
      );
      notifyPaths(path, subPaths, values);
    },
    [notifyPaths]
  );
  const notifyAll = useCallback(
    (values) => notifyPaths(createPxth([]), observersMap.current.keys(), values),
    [notifyPaths]
  );
  return {
    watch,
    watchAll,
    watchBatchUpdates,
    isObserved,
    notifySubTree,
    notifyAll
  };
}, "useObservers");

// src/hooks/useStock.ts
var useStock = /* @__PURE__ */ __name(({ initialValues, debugName }) => {
  const values = useLazyRef(() => cloneDeep(initialValues));
  const { notifySubTree, notifyAll, watch, watchAll, watchBatchUpdates, isObserved } = useObservers();
  const setValue = useCallback2(
    (path, action) => {
      const value = isFunction(action) ? action(deepGet2(values.current, path)) : action;
      values.current = deepSet(values.current, path, value);
      notifySubTree(path, values.current);
    },
    [values, notifySubTree]
  );
  const setValues = useCallback2(
    (newValues) => {
      values.current = newValues;
      notifyAll(newValues);
    },
    [values, notifyAll]
  );
  const getValue = useCallback2((path) => deepGet2(values.current, path), [values]);
  const getValues = useCallback2(() => values.current, [values]);
  const resetValues = useCallback2(() => setValues(cloneDeep(initialValues)), [initialValues, setValues]);
  const stock = useMemo(
    () => ({
      getValue,
      getValues,
      setValue,
      setValues,
      resetValues,
      watch,
      watchAll,
      watchBatchUpdates,
      isObserved,
      debugName,
      paths: createPxth2([])
    }),
    [
      getValue,
      getValues,
      debugName,
      setValue,
      setValues,
      resetValues,
      watch,
      watchAll,
      watchBatchUpdates,
      isObserved
    ]
  );
  useDebugStock(stock);
  return stock;
}, "useStock");

// src/hooks/useStockContext.ts
import { useContext } from "react";
import invariant4 from "tiny-invariant";

// src/components/StockContext.ts
import { createContext } from "react";
var StockContext = createContext(void 0);

// src/components/StockRoot.tsx
import React from "react";
var StockRoot = /* @__PURE__ */ __name((_a) => {
  var _b = _a, { children } = _b, stockConfig = __objRest(_b, ["children"]);
  const stock = useStock(stockConfig);
  return /* @__PURE__ */ React.createElement(StockContext.Provider, {
    value: stock
  }, children);
}, "StockRoot");

// src/components/ProxyContext.ts
import { createContext as createContext2 } from "react";
var ProxyContext = createContext2(void 0);

// src/components/ProxyProvider.tsx
import React2 from "react";
var ProxyProvider = /* @__PURE__ */ __name(({ proxy, children }) => {
  const stock = useStockContext();
  return /* @__PURE__ */ React2.createElement(StockContext.Provider, {
    value: stock
  }, /* @__PURE__ */ React2.createElement(ProxyContext.Provider, {
    value: proxy
  }, typeof children === "function" ? children(proxy.path) : children));
}, "ProxyProvider");

// src/utils/useInterceptors.ts
import { useCallback as useCallback3, useEffect as useEffect2 } from "react";
import cloneDeep2 from "lodash/cloneDeep";
import isFunction2 from "lodash/isFunction";
import unset from "lodash/unset";
import { deepGet as deepGet3, deepSet as deepSet2, getPxthSegments as getPxthSegments3, isInnerPxth as isInnerPxth2, samePxth as samePxth2 } from "pxth";
import invariant3 from "tiny-invariant";
var shouldUseProxy = /* @__PURE__ */ __name((proxy, path) => proxy && (isInnerPxth2(proxy.path, path) || samePxth2(proxy.path, path)), "shouldUseProxy");
var intercept = /* @__PURE__ */ __name((proxy, path, standardCallback, proxiedCallback, args) => {
  if (!shouldUseProxy(proxy, path)) {
    return standardCallback(...args);
  } else {
    return proxiedCallback(...args);
  }
}, "intercept");
var useInterceptors = /* @__PURE__ */ __name((stock, proxy) => {
  const { watch, setValue, getValue, setValues, getValues } = stock;
  useEffect2(
    () => invariant3(
      !proxy || proxy.isActive(),
      "Cannot use not activated proxy. Maybe you forgot to call proxy.activate()?"
    ),
    [proxy]
  );
  const interceptedWatch = useCallback3(
    (path, observer) => intercept(
      proxy,
      path,
      watch,
      (path2, observer2) => proxy.watch(path2, observer2, watch),
      [path, observer]
    ),
    [watch, proxy]
  );
  const interceptedSetValue = useCallback3(
    (path, value) => intercept(
      proxy,
      path,
      setValue,
      (path2, value2) => proxy.setValue(path2, value2, setValue, getValue),
      [path, value]
    ),
    [getValue, proxy, setValue]
  );
  const interceptedGetValue = useCallback3(
    (path) => intercept(proxy, path, getValue, (path2) => proxy.getValue(path2, getValue), [
      path
    ]),
    [proxy, getValue]
  );
  const interceptedGetValues = useCallback3(() => {
    let allValues = cloneDeep2(getValues());
    const proxiedValue = proxy.getValue(proxy.path, getValue);
    allValues = deepSet2(allValues, proxy.path, proxiedValue);
    return allValues;
  }, [proxy, getValues, getValue]);
  const interceptedSetValues = useCallback3(
    (values) => {
      const proxiedValue = deepGet3(values, proxy.path);
      unset(values, getPxthSegments3(proxy.path));
      proxy.setValue(
        proxy.path,
        proxiedValue,
        (path, value) => {
          values = deepSet2(values, path, isFunction2(value) ? value(deepGet3(values, path)) : value);
        },
        getValue
      );
      setValues(values);
    },
    [getValue, proxy, setValues]
  );
  if (!proxy) {
    return stock;
  }
  return __spreadProps(__spreadValues({}, stock), {
    watch: interceptedWatch,
    setValue: interceptedSetValue,
    getValue: interceptedGetValue,
    getValues: interceptedGetValues,
    setValues: interceptedSetValues
  });
}, "useInterceptors");

// src/hooks/useStockContext.ts
var useStockContext = /* @__PURE__ */ __name((stock, proxy) => {
  const contextStock = useContext(StockContext);
  const contextProxy = useContext(ProxyContext);
  stock = stock != null ? stock : contextStock;
  proxy = proxy != null ? proxy : contextProxy;
  invariant4(stock, "You're trying to access stock outside StockContext and without providing custom stock.");
  return useInterceptors(stock, proxy);
}, "useStockContext");

// src/hooks/useStockValue.ts
import { useEffect as useEffect3, useReducer } from "react";
var useStockValue = /* @__PURE__ */ __name((path, customStock, proxy) => {
  const stock = useStockContext(customStock, proxy);
  const { watch, getValue } = stock;
  const [, forceUpdate] = useReducer((val) => val + 1, 0);
  const value = useLazyRef(() => getValue(path));
  useEffect3(
    () => watch(path, (newValue) => {
      value.current = newValue;
      forceUpdate();
    }),
    [path, watch, value]
  );
  return value.current;
}, "useStockValue");

// src/hooks/useStockState.ts
import { useCallback as useCallback4 } from "react";
var useStockState = /* @__PURE__ */ __name((path, customStock, proxy) => {
  const stock = useStockContext(customStock, proxy);
  const value = useStockValue(path, customStock, proxy);
  const { setValue } = stock;
  const set = useCallback4((action) => setValue(path, action), [path, setValue]);
  return [value, set];
}, "useStockState");

// src/hooks/useAllStockValues.ts
import { createPxth as createPxth3 } from "pxth";
var rootPxth = createPxth3([]);
var useAllStockValues = /* @__PURE__ */ __name((customStock, proxy) => {
  return useStockValue(rootPxth, customStock, proxy);
}, "useAllStockValues");

// src/hooks/useMappingProxy.ts
import { useEffect as useEffect4, useMemo as useMemo2, useRef as useRef3 } from "react";

// src/typings/MappingProxy.ts
import isFunction3 from "lodash/isFunction";
import isNil from "lodash/isNil";
import { deepGet as deepGet4, deepSet as deepSet3, isInnerPxth as isInnerPxth4, joinPxths as joinPxths2, longestCommonPxth, relativePxth, samePxth as samePxth4 } from "pxth";
import invariant6 from "tiny-invariant";

// src/typings/StockProxy.ts
var StockProxy = class {
  constructor(path) {
    this.activate = /* @__PURE__ */ __name(() => Object.freeze(this), "activate");
    this.isActive = /* @__PURE__ */ __name(() => Object.isFrozen(this), "isActive");
    this.path = path;
  }
};
__name(StockProxy, "StockProxy");

// src/utils/createProxyMap.ts
import { createPxth as createPxth4, isPxth, joinPxths } from "pxth";
import invariant5 from "tiny-invariant";
var getAllObjectKeys = /* @__PURE__ */ __name((obj) => {
  if (Array.isArray(obj)) {
    return Object.keys(obj);
  }
  return [...Object.getOwnPropertyNames(obj), ...Object.getOwnPropertySymbols(obj)];
}, "getAllObjectKeys");
var createProxyMap = /* @__PURE__ */ __name((mapSource) => {
  const proxyMap = new PxthMap();
  if (isPxth(mapSource)) {
    proxyMap.set(createPxth4([]), mapSource);
    return proxyMap;
  }
  const queue = [[createPxth4([]), mapSource]];
  while (queue.length) {
    const [pathToObject, innerObject] = queue.shift();
    for (const key of getAllObjectKeys(innerObject)) {
      const item = innerObject[key];
      const pathToItem = joinPxths(pathToObject, createPxth4([key]));
      if (isPxth(item)) {
        proxyMap.set(pathToItem, item);
      } else if (typeof item === "object" && item !== null) {
        queue.push([pathToItem, item]);
      } else {
        invariant5(false, "Invalid proxy map passed.");
      }
    }
  }
  return proxyMap;
}, "createProxyMap");

// src/utils/mappingProxyUtils.ts
import { isInnerPxth as isInnerPxth3, samePxth as samePxth3 } from "pxth";
var getInnerPaths = /* @__PURE__ */ __name((path, proxyMap) => {
  return proxyMap.entries().filter(([to]) => {
    return isInnerPxth3(path, to) || samePxth3(path, to);
  });
}, "getInnerPaths");
var hasMappedParentPaths = /* @__PURE__ */ __name((path, proxyMap) => {
  return proxyMap.entries().some(([mappedPath]) => isInnerPxth3(mappedPath, path));
}, "hasMappedParentPaths");

// src/typings/MappingProxy.ts
var MappingProxy = class extends StockProxy {
  constructor(mapSource, path) {
    super(path);
    this.setValue = /* @__PURE__ */ __name((path, value, defaultSetValue, defaultGetValue) => {
      const relativeValuePath = relativePxth(this.path, path);
      const normalPath = this.getNormalPath(path);
      if (hasMappedParentPaths(relativeValuePath, this.proxyMap)) {
        defaultSetValue(normalPath, value);
        return;
      }
      const innerPaths = getInnerPaths(relativeValuePath, this.proxyMap);
      const oldValue = defaultGetValue(normalPath);
      innerPaths.forEach(
        ([to, from]) => from !== void 0 && defaultSetValue(
          from,
          deepGet4(isFunction3(value) ? value(oldValue) : value, relativePxth(relativeValuePath, to))
        )
      );
    }, "setValue");
    this.watch = /* @__PURE__ */ __name((path, observer, defaultWatch) => {
      const normalPath = this.getNormalPath(path);
      return defaultWatch(normalPath, (value) => observer(this.mapValue(value, path, normalPath)));
    }, "watch");
    this.getValue = /* @__PURE__ */ __name((path, defaultGetValue) => {
      const normalPath = this.getNormalPath(path);
      return this.mapValue(defaultGetValue(normalPath), path, normalPath);
    }, "getValue");
    this.mapValue = /* @__PURE__ */ __name((value, path, normalPath) => {
      path = relativePxth(this.path, path);
      if (hasMappedParentPaths(path, this.proxyMap)) {
        return value;
      }
      const innerPaths = getInnerPaths(path, this.proxyMap);
      return innerPaths.reduce(
        (acc, [to, from]) => deepSet3(
          acc,
          relativePxth(path, to),
          deepGet4(value, relativePxth(normalPath, from))
        ),
        {}
      );
    }, "mapValue");
    this.getProxiedPath = /* @__PURE__ */ __name((path) => {
      var _a;
      const normalPath = (_a = this.proxyMap.entries().find(([, from]) => samePxth4(from, path))) == null ? void 0 : _a[0];
      invariant6(
        !isNil(normalPath),
        "Mapping proxy error: trying to get normal path of proxied path, which is not defined in proxy map"
      );
      return joinPxths2(this.path, normalPath);
    }, "getProxiedPath");
    this.getNormalPath = /* @__PURE__ */ __name((path) => {
      const normalPath = relativePxth(this.path, path);
      const isIndependent = this.proxyMap.has(normalPath);
      if (isIndependent) {
        return this.proxyMap.get(normalPath);
      }
      const hasMappedChildrenPaths = this.proxyMap.keys().some((mappedPath) => isInnerPxth4(normalPath, mappedPath));
      if (hasMappedChildrenPaths) {
        return longestCommonPxth(
          this.proxyMap.entries().filter(([to]) => isInnerPxth4(normalPath, to)).map(([, from]) => from)
        );
      }
      if (hasMappedParentPaths(normalPath, this.proxyMap)) {
        const [to, from] = this.proxyMap.entries().find(([mappedPath]) => isInnerPxth4(mappedPath, normalPath));
        return joinPxths2(from, relativePxth(to, normalPath));
      }
      invariant6(false, "Mapping proxy error: trying to proxy value, which is not defined in proxy map.");
    }, "getNormalPath");
    this.proxyMap = createProxyMap(mapSource);
  }
};
__name(MappingProxy, "MappingProxy");

// src/utils/areProxyMapsEqual.ts
import { samePxth as samePxth5 } from "pxth";
var areProxyMapsEqual = /* @__PURE__ */ __name((map1, map2) => {
  const proxyMap1 = createProxyMap(map1);
  const proxyMap2 = createProxyMap(map2);
  if (proxyMap1.entries().length !== proxyMap2.entries().length) {
    return false;
  }
  for (const [key, value] of proxyMap1.entries()) {
    const value2 = proxyMap2.get(key);
    if (!proxyMap2.has(key) || !samePxth5(value, value2)) {
      return false;
    }
  }
  return true;
}, "areProxyMapsEqual");

// src/hooks/useMappingProxy.ts
var useMapMemo = /* @__PURE__ */ __name((map) => {
  const mapRef = useRef3(map);
  useEffect4(() => {
    if (!areProxyMapsEqual(mapRef.current, map)) {
      mapRef.current = map;
    }
  }, [map]);
  return areProxyMapsEqual(mapRef.current, map) ? mapRef.current : map;
}, "useMapMemo");
var useMappingProxy = /* @__PURE__ */ __name((mapSource, path) => {
  const realMap = useMapMemo(mapSource);
  return useMemo2(() => {
    const proxy = new MappingProxy(realMap, path);
    proxy.activate();
    return proxy;
  }, [hashPxth(path), realMap]);
}, "useMappingProxy");
export {
  MappingProxy,
  ObserverArray,
  ProxyContext,
  ProxyProvider,
  PxthMap,
  StockContext,
  StockProxy,
  StockRoot,
  intercept,
  useAllStockValues,
  useMappingProxy,
  useObservers,
  useStock,
  useStockContext,
  useStockState,
  useStockValue
};
